@@ -27,10 +27,13 @@
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include "gst-tool.h"
-#include "gst-widget.h"
 #include "gst-dialog.h"
 #include "gst-conf.h"
 
+#ifdef HAVE_POLKIT
+#include "gst-polkit-button.h"
+#endif
+
 #define GST_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GST_TYPE_DIALOG, GstDialogPrivate))
 
 typedef struct _GstDialogPrivate GstDialogPrivate;
@@ -41,15 +44,26 @@ struct _GstDialogPrivate {
 	gchar   *title;
 	gchar   *widget_name;
 
-	GladeXML  *gui;
-	GtkWidget *child;
+	GtkBuilder *builder;
+	GtkWidget  *child;
+#ifdef HAVE_POLKIT
+	GtkWidget  *polkit_button;
+#endif
+	GSList     *policy_list;
 
-	gboolean modified;
-	gboolean frozen;
+	guint    frozen;
+	guint    modified : 1;
+};
+
+struct _GstWidgetPolicy {
+	GtkWidget *widget;
+
+	gboolean need_access;
+	gboolean was_sensitive;
 };
 
 static void gst_dialog_class_init (GstDialogClass *class);
-static void gst_dialog_init       (GstDialog      *group);
+static void gst_dialog_init       (GstDialog      *dialog);
 static void gst_dialog_finalize   (GObject        *object);
 
 static GObject*	gst_dialog_constructor (GType                  type,
@@ -64,7 +78,16 @@ static void gst_dialog_set_property (GObject      *object,
 static void gst_dialog_response (GtkDialog *dialog,
 				 gint       response);
 
-static void gst_dialog_map      (GtkWidget *widget);
+static gboolean gst_dialog_delete_event (GtkWidget   *widget,
+					 GdkEventAny *event);
+static void     gst_dialog_realize      (GtkWidget   *widget);
+
+static void     gst_dialog_set_cursor   (GstDialog     *dialog,
+					 GdkCursorType  cursor_type);
+
+static void             gst_dialog_unlock                (GstDialog *dialog);
+static GstWidgetPolicy *gst_dialog_get_policy_for_widget (GstDialog *dialog,
+							  GtkWidget *widget);
 
 enum {
 	PROP_0,
@@ -73,6 +96,13 @@ enum {
 	PROP_TITLE
 };
 
+enum {
+	UNLOCKED,
+	LAST_SIGNAL
+};
+
+static guint signals [LAST_SIGNAL] = { 0 };
+
 G_DEFINE_TYPE (GstDialog, gst_dialog, GTK_TYPE_DIALOG);
 
 static void
@@ -87,7 +117,8 @@ gst_dialog_class_init (GstDialogClass *class)
 	object_class->finalize     = gst_dialog_finalize;
 
 	dialog_class->response     = gst_dialog_response;
-	widget_class->map          = gst_dialog_map;
+	widget_class->delete_event = gst_dialog_delete_event;
+	widget_class->realize      = gst_dialog_realize;
 
 	g_object_class_install_property (object_class,
 					 PROP_TOOL,
@@ -110,6 +141,16 @@ gst_dialog_class_init (GstDialogClass *class)
 							      "title",
 							      NULL,
 							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+	signals [UNLOCKED] =
+		g_signal_new ("unlocked",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GstDialogClass, unlocked),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
 	g_type_class_add_private (object_class,
 				  sizeof (GstDialogPrivate));
 }
@@ -118,19 +159,28 @@ static void
 gst_dialog_init (GstDialog *dialog)
 {
 	GstDialogPrivate *priv;
-	GtkWidget *button;
 
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
 
 	priv->tool  = NULL;
 	priv->title = NULL;
-	priv->gui   = NULL;
+	priv->builder = NULL;
 	priv->child = NULL;
 
 	priv->modified = FALSE;
-	priv->frozen   = FALSE;
 
-	dialog->gst_widget_list = NULL;
+	priv->policy_list = NULL;
+
+#ifdef HAVE_POLKIT
+	priv->polkit_button = gst_polkit_button_new (NULL, _("_Unlock"));
+	gtk_widget_show (priv->polkit_button);
+
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      priv->polkit_button, GTK_RESPONSE_NONE);
+
+	g_signal_connect_swapped (priv->polkit_button, "changed",
+				  G_CALLBACK (gst_dialog_unlock), dialog);
+#endif
 
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_HELP, GTK_RESPONSE_HELP,
@@ -155,10 +205,10 @@ gst_dialog_constructor (GType                  type,
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
 
 	if (priv->tool && priv->widget_name) {
-		priv->gui   = glade_xml_new (priv->tool->glade_path, NULL, NULL);
+		priv->builder = gtk_builder_new ();
 
-		if (!priv->gui) {
-			g_critical ("glade file not found: %s\n", priv->tool->glade_path);
+		if (!gtk_builder_add_from_file (priv->builder, priv->tool->ui_path, NULL)) {
+			g_critical ("UI file not found: %s\n", priv->tool->ui_path);
 
 			/* no point in continuing */
 			exit (-1);
@@ -167,7 +217,7 @@ gst_dialog_constructor (GType                  type,
 		toplevel = gtk_widget_get_toplevel (priv->child);
 
 		if (GTK_WIDGET_TOPLEVEL (priv->child)) {
-			g_critical ("The widget \"%s\" should not be a toplevel widget in the .glade file\n"
+			g_critical ("The widget \"%s\" should not be a toplevel widget in the .ui file\n"
 				    "You just need to add the widget inside a GtkWindow so that it can be deparented.", priv->widget_name);
 
 			/* no point in continuing */
@@ -181,6 +231,15 @@ gst_dialog_constructor (GType                  type,
 		gtk_widget_destroy (toplevel);
 	}
 
+#ifdef HAVE_POLKIT
+	if (priv->tool) {
+		const gchar *action;
+
+		action = oobs_session_get_authentication_action (priv->tool->session);
+		gst_polkit_button_set_action (GST_POLKIT_BUTTON (priv->polkit_button), action);
+	}
+#endif
+
 	if (priv->title)
 		gtk_window_set_title (GTK_WINDOW (dialog), priv->title);
 
@@ -216,8 +275,8 @@ gst_dialog_finalize (GObject *object)
 	GstDialog *dialog = GST_DIALOG (object);
 	GstDialogPrivate *priv = GST_DIALOG_GET_PRIVATE (dialog);
 
-	if (priv->gui)
-		g_object_unref (priv->gui);
+	if (priv->builder)
+		g_object_unref (priv->builder);
 
 	if (priv->child)
 		gtk_widget_destroy (priv->child);
@@ -225,9 +284,9 @@ gst_dialog_finalize (GObject *object)
 	g_free (priv->title);
 	g_free (priv->widget_name);
 
-	g_slist_foreach (dialog->gst_widget_list, (GFunc) g_free, NULL);
-	g_slist_free (dialog->gst_widget_list);
-	
+	g_slist_foreach (priv->policy_list, (GFunc) g_free, NULL);
+	g_slist_free (priv->policy_list);
+
 	(* G_OBJECT_CLASS (gst_dialog_parent_class)->finalize) (object);
 }
 
@@ -242,8 +301,11 @@ gst_dialog_response (GtkDialog *dialog,
 	switch (response) {
 	case GTK_RESPONSE_CLOSE:
 	case GTK_RESPONSE_DELETE_EVENT:
-		gtk_widget_hide (GTK_WIDGET (dialog));
-		gst_tool_close (priv->tool);
+		if (GTK_WIDGET_SENSITIVE (dialog)) {
+			gtk_widget_hide (GTK_WIDGET (dialog));
+			gst_tool_close (priv->tool);
+		}
+
 		break;
 	case GTK_RESPONSE_HELP:
 		gst_tool_show_help (priv->tool, NULL);
@@ -252,18 +314,24 @@ gst_dialog_response (GtkDialog *dialog,
 }
 
 static void
-gst_dialog_map (GtkWidget *widget)
+gst_dialog_realize (GtkWidget *widget)
 {
-	GstDialog *dialog = GST_DIALOG (widget);
 	GstDialogPrivate *priv;
 
-	(* GTK_WIDGET_CLASS (gst_dialog_parent_class)->map) (widget);
+	(* GTK_WIDGET_CLASS (gst_dialog_parent_class)->realize) (widget);
 
-	priv = GST_DIALOG_GET_PRIVATE (dialog);
+	priv = GST_DIALOG_GET_PRIVATE (widget);
 
-	gst_dialog_freeze_visible (dialog);
-	gst_tool_update_gui (priv->tool);
-	gst_dialog_thaw_visible (dialog);
+	if (priv->frozen > 0)
+		gst_dialog_set_cursor (GST_DIALOG (widget), GDK_WATCH);
+}
+
+static gboolean
+gst_dialog_delete_event (GtkWidget   *widget,
+			 GdkEventAny *event)
+{
+	/* we don't want the dialog to be destroyed at this point */
+	return TRUE;
 }
 
 GstDialog*
@@ -288,9 +356,9 @@ gst_dialog_get_widget (GstDialog *dialog, const char *widget)
 
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
 
-	g_return_val_if_fail (priv->gui != NULL, NULL);
+	g_return_val_if_fail (priv->builder != NULL, NULL);
 
-	w = glade_xml_get_widget (priv->gui, widget);
+	w = GTK_WIDGET (gtk_builder_get_object (priv->builder, widget));
 
 	if (!w)
 		g_error ("Could not find widget: %s", widget);
@@ -298,18 +366,34 @@ gst_dialog_get_widget (GstDialog *dialog, const char *widget)
 	return w;
 }
 
-void
-gst_dialog_apply_widget_policies (GstDialog *dialog)
+static void
+gst_dialog_unlock (GstDialog *dialog)
 {
 	GstDialogPrivate *priv;
 	GSList *list;
 
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
 
-	/* Hide, show + desensitize or show + sensitize widgets based on access level */
-	for (list = dialog->gst_widget_list; list; list = g_slist_next (list)) {
-		gst_widget_apply_policy (list->data);
+	for (list = priv->policy_list; list; list = g_slist_next (list)) {
+		GstWidgetPolicy *policy;
+		policy = (GstWidgetPolicy *) list->data;
+		gtk_widget_set_sensitive (policy->widget, policy->was_sensitive);
 	}
+
+	g_signal_emit (dialog, signals [UNLOCKED], 0);
+}
+
+static void
+gst_dialog_set_cursor (GstDialog     *dialog,
+		       GdkCursorType  cursor_type)
+{
+	GdkCursor *cursor;
+
+	g_return_if_fail (GTK_WIDGET_REALIZED (dialog));
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_window_set_cursor (GTK_WIDGET (dialog)->window, cursor);
+	gdk_cursor_unref (cursor);
 }
 
 void
@@ -317,65 +401,50 @@ gst_dialog_freeze (GstDialog *dialog)
 {
 	GstDialogPrivate *priv;
 
-	g_return_if_fail (dialog != NULL);
 	g_return_if_fail (GST_IS_DIALOG (dialog));
 
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
-	priv->frozen = TRUE;
+
+	if (priv->frozen == 0) {
+		if (GTK_WIDGET (dialog)->window)
+			gst_dialog_set_cursor (dialog, GDK_WATCH);
+
+		gtk_widget_set_sensitive (GTK_WIDGET (dialog), FALSE);
+	}
+
+	priv->frozen++;
 }
 
-void 
+void
 gst_dialog_thaw (GstDialog *dialog)
 {
 	GstDialogPrivate *priv;
 
-	g_return_if_fail (dialog != NULL);
-	g_return_if_fail (GST_IS_DIALOG (dialog));
-
-	priv = GST_DIALOG_GET_PRIVATE (dialog);
-	priv->frozen = FALSE;
-}
-
-void
-gst_dialog_freeze_visible (GstDialog *dialog)
-{
-	GstDialogPrivate *priv;
-	GdkCursor *cursor;
-	
-	g_return_if_fail (dialog != NULL);
 	g_return_if_fail (GST_IS_DIALOG (dialog));
 
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
 
-	if (GTK_WIDGET (dialog)->window) {
-		cursor = gdk_cursor_new (GDK_WATCH);
-		gdk_window_set_cursor (GTK_WIDGET (dialog)->window, cursor);
-		gdk_cursor_unref (cursor);
-	}
+	g_return_if_fail (priv->frozen != 0);
 
-	if (!priv->frozen)
-		gtk_widget_set_sensitive (GTK_WIDGET (dialog), FALSE);
-	
-	gst_dialog_freeze (dialog);
-}
+	priv->frozen--;
 
-void
-gst_dialog_thaw_visible (GstDialog *dialog)
-{
-	GstDialogPrivate *priv;
+	if (priv->frozen == 0) {
+		if (GTK_WIDGET (dialog)->window)
+			gdk_window_set_cursor (GTK_WIDGET (dialog)->window, NULL);
 
-	g_return_if_fail (dialog != NULL);
-	g_return_if_fail (GST_IS_DIALOG (dialog));
-
-	priv = GST_DIALOG_GET_PRIVATE (dialog);
-
-	if (GTK_WIDGET (dialog)->window)
-		gdk_window_set_cursor (GTK_WIDGET (dialog)->window, NULL);
-
-	gst_dialog_thaw (dialog);
-
-	if (!priv->frozen)
 		gtk_widget_set_sensitive (GTK_WIDGET (dialog), TRUE);
+	}
+}
+
+guint
+gst_dialog_get_freeze_level (GstDialog *dialog)
+{
+	GstDialogPrivate *priv;
+
+	g_return_if_fail (GST_IS_DIALOG (dialog));
+
+	priv = GST_DIALOG_GET_PRIVATE (dialog);
+	return priv->frozen;
 }
 
 gboolean
@@ -412,7 +481,7 @@ gst_dialog_modify (GstDialog *dialog)
 
 	priv = GST_DIALOG_GET_PRIVATE (dialog);
 
-	if (priv->frozen || !gst_tool_is_authenticated (priv->tool))
+	if (priv->frozen || !gst_dialog_is_authenticated (dialog))
 		return;
 
 	gst_dialog_set_modified (dialog, TRUE);
@@ -424,62 +493,6 @@ gst_dialog_modify_cb (GtkWidget *w, gpointer data)
 	gst_dialog_modify (data);
 }
 
-void
-gst_dialog_set_widget_policies (GstDialog *dialog, const GstWidgetPolicy *wp)
-{
-	GstWidget *w;
-	int i;
-
-	for (i = 0; wp [i].widget; i++) {
-		w = gst_widget_new (dialog, wp [i]);
-	}
-}
-
-void
-gst_dialog_set_widget_user_modes (GstDialog *xd, const GstWidgetUserPolicy *xwup)
-{
-	GstWidget *xw;
-	int i;
-
-	for (i = 0; xwup [i].widget; i++)
-	{
-		xw = gst_dialog_get_gst_widget (xd, xwup [i].widget);
-
-		if (!xw)
-			xw = gst_widget_new_full (gst_dialog_get_widget (xd, xwup [i].widget), xd,
-						  GST_WIDGET_MODE_SENSITIVE, FALSE, TRUE);
-
-		gst_widget_set_user_mode (xw, xwup [i].mode);
-	}
-
-	gst_dialog_apply_widget_policies (xd);
-}
-
-GstWidget *
-gst_dialog_get_gst_widget (GstDialog *xd, const gchar *name)
-{
-	GstWidget *xw = NULL;
-	GtkWidget *widget;
-	GSList *list;
-
-	g_return_val_if_fail (xd != NULL, NULL);
-
-	widget = gst_dialog_get_widget (xd, name);
-
-	for (list = xd->gst_widget_list; list; list = g_slist_next (list))
-	{
-		if (((GstWidget *) list->data)->widget == widget)
-		{
-			xw = list->data;
-			break;
-		}
-	}
-
-	if (xw == NULL)
-		g_warning ("Widget %s not found in policy table.", name);
-	return (xw);
-}
-
 GstTool*
 gst_dialog_get_tool (GstDialog *dialog)
 {
@@ -489,30 +502,78 @@ gst_dialog_get_tool (GstDialog *dialog)
 	return priv->tool;
 }
 
-void
-gst_dialog_widget_set_user_mode (GstDialog *xd, const gchar *name, GstWidgetMode mode)
+static GstWidgetPolicy *
+gst_dialog_get_policy_for_widget (GstDialog *dialog, GtkWidget *widget)
 {
-	GstWidget *xw;
+	GstDialogPrivate *priv;
+	GstWidgetPolicy *policy = NULL;
+	GSList *list;
 
-	g_return_if_fail (xd != NULL);
+	g_return_val_if_fail (GST_IS_DIALOG (dialog), NULL);
 
-	xw = gst_dialog_get_gst_widget (xd, name);
-	g_return_if_fail (xw != NULL);
-	
-	gst_widget_set_user_mode (xw, mode);
+	priv = GST_DIALOG_GET_PRIVATE (dialog);
+
+	for (list = priv->policy_list; list; list = g_slist_next (list)) {
+		if (((GstWidgetPolicy *) list->data)->widget == widget) {
+			policy = list->data;
+			break;
+		}
+	}
+
+	return policy;
 }
 
 void
-gst_dialog_widget_set_user_sensitive (GstDialog *xd, const gchar *name, gboolean state)
+gst_dialog_require_authentication_for_widget (GstDialog *xd, GtkWidget *w)
 {
-	GstWidget *xw;
+	GstDialogPrivate *priv;
+	GstWidgetPolicy *p;
 
-	g_return_if_fail (xd != NULL);
-	
-	xw = gst_dialog_get_gst_widget (xd, name);
-	g_return_if_fail (xw != NULL);
+	priv = GST_DIALOG_GET_PRIVATE (xd);
+	p = g_new0 (GstWidgetPolicy, 1);
 
-	gst_widget_set_user_sensitive (xw, state);
+	p->widget = w;
+	p->was_sensitive = GTK_WIDGET_SENSITIVE (w);
+
+	priv->policy_list = g_slist_prepend (priv->policy_list, p);
+
+	gtk_widget_set_sensitive (w, gst_dialog_is_authenticated (xd));
+}
+
+void
+gst_dialog_require_authentication_for_widgets (GstDialog *xd, const gchar **names)
+{
+	g_return_if_fail (GST_IS_DIALOG (xd));
+
+	for (; *names != NULL; names++) {
+		GstDialogPrivate *priv;
+		GtkWidget *w;
+
+		priv = GST_DIALOG_GET_PRIVATE (xd);
+		w = gst_dialog_get_widget (xd, *names);
+		g_return_if_fail (w != NULL);
+
+		gst_dialog_require_authentication_for_widget (xd, w);
+	}
+}
+
+void
+gst_dialog_try_set_sensitive (GstDialog *dialog, GtkWidget *w, gboolean sensitive)
+{
+	g_return_if_fail (GST_IS_DIALOG (dialog));
+
+	if (sensitive) {
+		GstWidgetPolicy *policy;
+
+		policy = gst_dialog_get_policy_for_widget (dialog, w);
+
+		if (policy) {
+			policy->was_sensitive = TRUE;
+			sensitive = gst_dialog_is_authenticated (dialog);
+		}
+	}
+
+	gtk_widget_set_sensitive (w, sensitive);
 }
 
 static void
@@ -557,3 +618,17 @@ gst_dialog_connect_signals_after (GstDialog *dialog, GstDialogSignal *signals)
 {
 	dialog_connect_signals (dialog, signals, TRUE);
 }
+
+gboolean
+gst_dialog_is_authenticated (GstDialog *dialog)
+{
+#ifdef HAVE_POLKIT
+	GstDialogPrivate *priv;
+
+	priv = GST_DIALOG_GET_PRIVATE (dialog);
+
+	return gst_polkit_button_get_authenticated (GST_POLKIT_BUTTON (priv->polkit_button));
+#else
+	return TRUE;
+#endif
+}
