@@ -25,7 +25,6 @@
 
 G_BEGIN_DECLS
 
-#include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <oobs/oobs.h>
 #include <gconf/gconf-client.h>
@@ -49,10 +48,11 @@ struct _GstTool {
 	gchar *icon;
 
 	OobsSession *session;
+	GPtrArray   *objects;
 	GConfClient *gconf_client;
 
-	char *glade_path;
-	char *glade_common_path;
+	char *ui_path;
+	char *common_ui_path;
 
 	GtkIconTheme *icon_theme;
 	GstDialog *main_dialog;
@@ -82,7 +82,6 @@ GType        gst_tool_get_type            (void);
 void         gst_init                     (const gchar *app_name,
 					   int argc, char *argv [],
 					   GOptionEntry *entries);
-gboolean     gst_tool_is_authenticated    (GstTool *tool);
 
 void         gst_tool_update_gui          (GstTool *tool);
 void         gst_tool_update_config       (GstTool *tool);
@@ -99,6 +98,12 @@ void         gst_tool_commit_async    (GstTool             *tool,
 				       OobsObjectAsyncFunc  func,
 				       gpointer             data);
 
+void         gst_tool_update_async    (GstTool             *tool);
+
+void         gst_tool_add_configuration_object (GstTool    *tool,
+						OobsObject *object);
+
+
 G_END_DECLS
 
 #endif /* __GST_TOOL_H */
