@@ -54,7 +54,7 @@ check_user_delete (OobsUser *user)
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
-						 _("Administrator account can not be deleted"));
+						 _("Administrator account cannot be deleted"));
 
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 							  _("This would leave the system unusable."));
@@ -64,8 +64,6 @@ check_user_delete (OobsUser *user)
 		return FALSE;
 	}
 
-	/* FIXME: could check that user is logged in */
-
 	dialog = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
 					 GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_WARNING,
@@ -75,6 +73,31 @@ check_user_delete (OobsUser *user)
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  _("This will disable this user's access to the system "
 						    "without deleting the user's home directory."));
+
+	if (oobs_user_get_active (user)) {
+		GtkWidget *alignment, *box, *image, *label;
+		gint image_width;
+
+		image = gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU);
+		gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, &image_width, NULL);
+
+		label = gtk_label_new (_("This user is currently using this computer"));
+		gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
+
+		box = gtk_hbox_new (FALSE, 12);
+		gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+		alignment = gtk_alignment_new (0., 0., 0., 0.);
+		gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, image_width, 0);
+		gtk_container_add (GTK_CONTAINER (alignment), box);
+		gtk_widget_show_all (alignment);
+
+
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), alignment, FALSE, FALSE, 0);
+	}
+
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_DELETE, GTK_RESPONSE_ACCEPT,
@@ -146,7 +169,7 @@ set_main_group (OobsUser *user)
 	main_group = oobs_user_get_main_group (user);
 
 	if (!main_group)
-		main_group = oobs_users_config_get_default_group (GST_USERS_TOOL (tool)->users_config);
+		main_group = oobs_users_config_get_default_group (OOBS_USERS_CONFIG (GST_USERS_TOOL (tool)->users_config));
 
 	combo = gst_dialog_get_widget (tool->main_dialog, "user_settings_group");
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
@@ -442,7 +465,7 @@ check_login (gchar **primary_text, gchar **secondary_text, gpointer data)
 		*primary_text = g_strdup (_("User name is empty"));
 		*secondary_text = g_strdup (_("A user name must be specified."));
 	} else if (is_user_root (user) && strcmp (login, "root") != 0) {
-		*primary_text = g_strdup (_("User name of the administrator account should not be modified"));
+		*primary_text = g_strdup (_("Administrator account's user name should not be modified"));
 	        *secondary_text = g_strdup (_("This would leave the system unusable."));
 	} else if (!is_valid_name (login)) {
 		*primary_text = g_strdup (_("User name has invalid characters"));
@@ -512,7 +535,7 @@ check_uid (gchar **primary_text, gchar **secondary_text, gpointer data)
 	uid = gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget));
 
 	if (is_user_root (user) && uid != 0) {
-		*primary_text   = g_strdup (_("User ID of the administrator account should not be modified"));
+		*primary_text   = g_strdup (_("Administrator account's user ID should not be modified"));
 		*secondary_text = g_strdup (_("This would leave the system unusable."));
 	}
 }
@@ -541,17 +564,27 @@ check_password (gchar **primary_text, gchar **secondary_text, gpointer data)
 	GtkWidget *widget;
 	const gchar *password, *confirmation;
 
-	widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_passwd1");
-	password = gtk_entry_get_text (GTK_ENTRY (widget));
+	widget = gst_dialog_get_widget (tool->main_dialog, "user_passwd_manual");
 
-	widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_passwd2");
-	confirmation = gtk_entry_get_text (GTK_ENTRY (widget));
+	/* manual password? */
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_passwd1");
+		password = gtk_entry_get_text (GTK_ENTRY (widget));
 
-	if (strlen (password) < 1) {
-		*primary_text = g_strdup (_("Password should not be empty"));
-		*secondary_text = g_strdup (_("A password must be provided."));
+		widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_passwd2");
+		confirmation = gtk_entry_get_text (GTK_ENTRY (widget));
+	}
+	else {
+		widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_random_passwd");
+		password = confirmation = gtk_entry_get_text (GTK_ENTRY (widget));
+	}
+
+	if (strlen (password) < 6) {
+		*primary_text = g_strdup (_("Password is too short"));
+		*secondary_text = g_strdup (_("User passwords must be longer than 6 characters and preferably "
+					      "formed by numbers, letters and special characters."));
 	} else if (strcmp (password, confirmation) != 0) {
-		*primary_text = g_strdup (_("Password confirmation isn't correct"));
+		*primary_text = g_strdup (_("Password confirmation is not correct"));
 		*secondary_text = g_strdup (_("Check that you have provided the same password in both text fields."));
 	}
 }
